/*  $Id: EventCounter.java 1338 2012-08-25 17:38:30Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import java.util.Hashtable;


/** 
 * Maintain abstract information regarding objects or event handling.
 * The status counter is computed from refular expression applied to string
 * serialisation of objects to be handled. The counter can be for each event
 * incremented, decremented, reseted or kept as is.<br>
 * <b>warning:</b> this class is not thread safe. Regular use steps are :
 * - 1: setup (incoke constructor)
 * - 2: config (invoke addMask, setMin, setMax, ...)
 * - 3: provide and register this handler to a handler process string.
 */
public class EventCounter<T> implements Handler<T>, Observable<EventCounter<T>>
{
    /** 
     * Add matching pattern linked to an operation on counter.
     * @param regexStr regular expression defining the matching rule.
     * @param opCode operation link to the matching rule.
     */ 
    public void addMasks(String regexStr,EOpCode opCode) 
                                throws PatternSyntaxException {
        Pattern pattern=Pattern.compile(regexStr);
        _masks.put(regexStr,pattern);
        _opCodes.put(pattern,opCode);
    }

    /** 
     * Remove all matching rules.
     * <b>warning:</b> use with caution, this method is not thread safe upon Handler
     * interface implementation.
     */ 
    public void resetMasks()
    {
        _masks=new Hashtable<String,Pattern>();
        _opCodes=new Hashtable<Pattern,EOpCode>();
    }

    // --- Handler implementation ---
    public void handle(T toHandle) {
        for(Pattern pat : _masks.values()) {
            if (pat.matcher(toHandle.toString()).matches()) {
                switch (_opCodes.get(pat)) {
                    case RESET:
                        _value=0;
                        break;
                    case INC:
                        _value++;
                        if (_value>_max) {
                            _value=_max;
                        }
                        break;
                    case DEC:
                        _value--;
                        if (_value<_min) {
                            _value=_min;
                        }
                        break;				
                }
                _obsManager.notify(this);
            }
        }
    }
    /** status counter value */
    private int _value=0;
    /**
     * get current value of the status counter.
     * @return current count.
     */ 
    public int getValue() {
        return _value;
    }
    /** min value for the status counter */
    private int _min=0;
    /** 
     * Set min value for the status counter.
     * @param m new min value
     */ 
    public void setMin(int m) {
        _min=m;
    }
    /** max value for the status counter */
    private int _max=1;
    /** 
     * Set max value for status counter.
     * @param m new max value
     */ 
    public void setMax(int m) {
        _max=m;
    }
    // --- Observable implementation ---
    private ObserverManager<EventCounter<T>> _obsManager=
                        new ObserverManager<EventCounter<T>>();
    // --- Observable implementation ---
    public ObserverManager<EventCounter<T>> getObserverManager() {
        return _obsManager;
    }
    /** 
     * Matching rules
     * @djin.relation composition,-,*,Pattern
     */ 
    private Hashtable<String,Pattern> _masks=
                                        new Hashtable<String,Pattern>();
    /** 
     * Matching rules to opcode links. 
     */
    private Hashtable<Pattern,EOpCode> _opCodes=
                           new Hashtable<Pattern,EOpCode>();
    /** Operation code applicable to the status counter */
    public enum EOpCode { RESET, INC, DEC };
}
